package bensley.andmote;

import java.util.Timer;
import java.util.TimerTask;

import android.content.Context;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;

public class InputControl extends View implements OnTouchListener {

	// Declare an empty timer object and a tick value in milliseconds. 150 isn't very fast, I prefer at least 100
	// but having a server end written in VB6 is going to help speed things up :)
	// There is an "On Error Resume Next" statement in the VB server because if this java client is sending the 
	// X and Y co-ords every 100ms, the VB end hasn't processed the results quick enough and the string buffer at 
	// the VB end that stores data received over the socket ends up containing "x=1234y=1234x=5678y=5678), 
	// i.e. two sets of X and Y values. Even at 150ms and occasionally at 200ms this will happen, so I have left the
	// statement in the server end and brought this down to 150ms allowing for the odd error, but if say you set 
	// it to 50ms there would be so many errors the cursor wouldn't move hardly anywhere. 
	// For this reason I added the sensitivity options which can be controlled with the volume keys to counteract
	// the slowed down send rate
	public static Timer TMR_WAIT;
	public long lngTick = 150;
	
	// Declare 4 Longs, the current X position (fltCurX) and where it was last time the onTouch method was called 
	// (fltLastX) the same for the Y axis (fltCurY and fltLastY). Once the user puts their finger on the screen the 
	// onTouch() method is called, this will move what's currently in fltCurX and fltCurY into fltLastX and fltLastY.
	// Then put the current X and Y co-ords in to fltCurX and fltCurY.
	// When the timer in this class 'ticks' it will send the values of (fltCurX-fltLastX) and (fltCurY-fltLastY) which
	// will essentially be the difference to where the finger is now to where it was last, giving a difference 
	// in location which at the server end can be added/subtracted to the current cursor position to give movement
	public float fltCurX = 0;
	public float fltLastX;
	public float fltCurY = 0;
	public float fltLastY;
	
	// These 3 variables are for sensing a mouse click. We can't send a mouse click every time the onTouch method is
	// called because its called when we want to move the cursor (that could get annoying!). So when onTouch is called
	// like the axis longs, the time the touch event took place is stored in lngCurDown (and what was in there is moved
	//  to lngLastDown before hand). If the difference between lngLastDown and lngCurDown is < lngClickDelay (a value 
	// in milliseconds) we send a single click to the server. 
	// So to double click, touch the screen thrice within 2*lngClickDelay
	public long lngClickDelay = 200;
	public long lngLastDown;
	public long lngCurDown = 0;
	
    public InputControl(Context context) {
        super(context);
        setFocusable(true); // Set this view extension to focusable (so we can capture input)
        setFocusableInTouchMode(true); // setFocusableInTouchMode so we can capture all touch and key events including volume up and down
        // However, this includes the Back back so later on we have to parse on this event to make sure it is received by super() so it
        // can perform its normal duty of going back to our main activity
        this.setOnTouchListener(this); // Define the touch listener for this view as within this class
    }

    public boolean onTouch(View view, MotionEvent event) {
    	fltLastX = fltCurX;
    	fltLastY = fltCurY;
    	fltCurX = event.getX();
    	fltCurY = event.getY();

    	if(event.getAction() == MotionEvent.ACTION_DOWN){
        	lngLastDown = lngCurDown;
        	lngCurDown = event.getDownTime();
        	// Start the timer to poll for X,Y co-ords and start sending them to the server as soon as we start touching the screen
        	durWaitSchedule("start");
        	// If this is the second touch within lngClickDelay duration, send a single click event to the server
        	if(lngCurDown-lngLastDown<=lngClickDelay){
        		ClientSocket.strTCPData = "singleclick";
        		CSNotify();
        	}
        }
        
        if(event.getAction() == MotionEvent.ACTION_UP){
        	// As soon as touch is released from the screen, stop sending X,Y co-ords by stopping the timer that polls for co-ords
        	durWaitSchedule("stop");
        }
           	
        return true;
    }
    
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
    	// The volume up and down keys are used to changed sensitivity at the server end, capture them here
    	if(keyCode==KeyEvent.KEYCODE_VOLUME_UP) {
    		// For each volume key we are just parsing a string to the Socket class and calling the method to notify the thread to wake up
    		// and send the pending string
    		ClientSocket.strTCPData = "volup";
    		CSNotify();
    		// We then return true to stop our button press being caught by super and actually being processed
    		// (there by turning the volume up and down)
    		return true;
        }else if(keyCode==KeyEvent.KEYCODE_VOLUME_DOWN) {
        	ClientSocket.strTCPData = "voldown";
        	CSNotify();
        	return true;
        }else{
        	// We don't want to do affect any other buttons, so return and let super get this event if it wasn't a volume button
        	super.onKeyDown(keyCode, event);
        }
    	return false;
    }

    public void durWaitSchedule(String action) {
    	if (action=="start") { // If we have been parsed the string "start", 
    		InputControl.TMR_WAIT = new Timer(); // create a new time and set it ticking at our lngTick interval
    		InputControl.TMR_WAIT.schedule(new durWaitTask(), lngTick, lngTick);
    	}else if (action=="stop") { // Else, cancel an existing timer
    		InputControl.TMR_WAIT.cancel();
    	}
    }

    class durWaitTask extends TimerTask
    {
        @Override
        public void run()
        {
        	// Remove comments and include android.util.Log to see X and Y movements
        	//Log.d("CurX: " + String.valueOf(fltCurX), "LastX: " + String.valueOf(fltLastX));
        	//Log.d("CurY: " + String.valueOf(fltCurY), "LastY: " + String.valueOf(fltLastY));
        	//Log.d("DifX: " + String.valueOf(fltCurX-fltLastX), "DifY: " + String.valueOf(fltCurY-fltLastY));
        	ClientSocket.strTCPData = (String.valueOf(fltCurX-fltLastX) + "," + String.valueOf(fltCurY-fltLastY));
        	Input.CSNotify();
        }
    }
    
    public void CSNotify() {
    	// Call the method within the Input class to notify the worker thread to wake up because there is probably some data waiting to be sent
    	Input.CSNotify();
    }

}